@@ -45,11 +45,13 @@ ADDRTYPE_P2WPKH = 6
 XPRV_HEADER = 0x0488ade4
 XPUB_HEADER = 0x0488b21e
 HEADERS_URL = "https://headers.electrum.org/blockchain_headers"
+GENESIS = "000000000019d6689c085ae165831e934ff763ae46a2a6c172b3f1b60a8ce26f"
 
 def set_testnet():
     global ADDRTYPE_P2PKH, ADDRTYPE_P2SH, ADDRTYPE_P2WPKH
     global XPRV_HEADER, XPUB_HEADER
     global TESTNET, HEADERS_URL
+    global GENESIS
     TESTNET = True
     ADDRTYPE_P2PKH = 111
     ADDRTYPE_P2SH = 196
@@ -57,18 +59,21 @@ def set_testnet():
     XPRV_HEADER = 0x04358394
     XPUB_HEADER = 0x043587cf
     HEADERS_URL = "https://headers.electrum.org/testnet_headers"
+    GENESIS = "000000000933ea01ad0ee984209779baaec3ced90fa3f408719526f8d77f4943"
 
 def set_nolnet():
     global ADDRTYPE_P2PKH, ADDRTYPE_P2SH, ADDRTYPE_P2WPKH
     global XPRV_HEADER, XPUB_HEADER
     global NOLNET, HEADERS_URL
-    NOLNET = True
+    global GENESIS
+    TESTNET = True
     ADDRTYPE_P2PKH = 0
     ADDRTYPE_P2SH = 5
     ADDRTYPE_P2WPKH = 6
     XPRV_HEADER = 0x0488ade4
     XPUB_HEADER = 0x0488b21e
     HEADERS_URL = "https://headers.electrum.org/nolnet_headers"
+    GENESIS = "663c88be18d07c45f87f910b93a1a71ed9ef1946cad50eb6a6f3af4c424625c6"
 
 
 
@@ -481,14 +486,14 @@ def msg_magic(message):
 
 def verify_message(address, sig, message):
     try:
-        public_key, compressed = pubkey_from_signature(sig, message)
+        h = Hash(msg_magic(message))
+        public_key, compressed = pubkey_from_signature(sig, h)
         # check public key using the address
         pubkey = point_to_ser(public_key.pubkey.point, compressed)
         addr = public_key_to_p2pkh(pubkey)
         if address != addr:
             raise Exception("Bad signature")
         # check message
-        h = Hash(msg_magic(message))
         public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
         return True
     except Exception as e:
@@ -570,7 +575,7 @@ class MyVerifyingKey(ecdsa.VerifyingKey):
         return klass.from_public_point( Q, curve )
 
 
-def pubkey_from_signature(sig, message):
+def pubkey_from_signature(sig, h):
     if len(sig) != 65:
         raise Exception("Wrong encoding")
     nV = ord(sig[0])
@@ -582,7 +587,6 @@ def pubkey_from_signature(sig, message):
     else:
         compressed = False
     recid = nV - 27
-    h = Hash(msg_magic(message))
     return MyVerifyingKey.from_signature(sig[1:], recid, h, curve = SECP256k1), compressed
 
 
@@ -631,12 +635,12 @@ class EC_KEY(object):
 
 
     def verify_message(self, sig, message):
-        public_key, compressed = pubkey_from_signature(sig, message)
+        h = Hash(msg_magic(message))
+        public_key, compressed = pubkey_from_signature(sig, h)
         # check public key
         if point_to_ser(public_key.pubkey.point, compressed) != point_to_ser(self.pubkey.point, compressed):
             raise Exception("Bad signature")
         # check message
-        h = Hash(msg_magic(message))
         public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
 
 
@@ -816,6 +820,21 @@ def xpub_from_pubkey(xtype, cK):
     return serialize_xpub(xtype, chr(0)*32, cK)
 
 
+def bip32_derivation(s):
+    assert s.startswith('m/')
+    s = s[2:]
+    for n in s.split('/'):
+        if n == '': continue
+        i = int(n[:-1]) + BIP32_PRIME if n[-1] == "'" else int(n)
+        yield i
+
+def is_bip32_derivation(x):
+    try:
+        [ i for i in bip32_derivation(x)]
+        return True
+    except :
+        return False
+
 def bip32_private_derivation(xprv, branch, sequence):
     assert sequence.startswith(branch)
     if branch == sequence:
