@@ -284,15 +284,13 @@ def match_decoded(decoded, to_match):
 
 
 def parse_sig(x_sig):
-    s = []
-    for sig in x_sig:
-        if sig[-2:] == '01':
-            s.append(sig[:-2])
-        else:
-            assert sig == NO_SIGNATURE
-            s.append(None)
-    return s
+    return map(lambda x: None if x == NO_SIGNATURE else x, x_sig)
 
+def safe_parse_pubkey(x):
+    try:
+        return xpubkey_to_pubkey(x)
+    except:
+        return x
 
 
 def parse_scriptSig(d, bytes):
@@ -335,9 +333,7 @@ def parse_scriptSig(d, bytes):
         try:
             signatures = parse_sig([sig])
             pubkey, address = xpubkey_to_address(x_pubkey)
-        except:
-            import traceback
-            traceback.print_exc(file=sys.stdout)
+        except BaseException:
             print_error("cannot find address in input script", bytes.encode('hex'))
             return
         d['type'] = 'p2pkh'
@@ -364,7 +360,7 @@ def parse_scriptSig(d, bytes):
         print_error("cannot find address in input script", bytes.encode('hex'))
         return
     x_pubkeys = map(lambda x: x[1].encode('hex'), dec2[1:-2])
-    pubkeys = [xpubkey_to_pubkey(x) for x in x_pubkeys]
+    pubkeys = [safe_parse_pubkey(x) for x in x_pubkeys]
     redeemScript = multisig_script(pubkeys, m)
     # write result in d
     d['type'] = 'p2sh'
@@ -417,9 +413,12 @@ def parse_input(vds):
     if prevout_hash == '00'*32:
         d['type'] = 'coinbase'
     else:
+        d['x_pubkeys'] = []
         d['pubkeys'] = []
         d['signatures'] = {}
         d['address'] = None
+        d['type'] = 'unknown'
+        d['num_sig'] = 0
         if scriptSig:
             parse_scriptSig(d, scriptSig)
     return d
@@ -480,6 +479,7 @@ def get_scriptPubKey(addr):
     return script
 
 def segwit_script(pubkey):
+    pubkey = safe_parse_pubkey(pubkey)
     pkh = hash_160(pubkey.decode('hex')).encode('hex')
     return '00' + push_script(pkh)
 
@@ -514,6 +514,7 @@ class Transaction:
         self._inputs = None
         self._outputs = None
         self.locktime = 0
+        self.version = 1
 
     def update(self, raw):
         self.raw = raw
@@ -555,7 +556,7 @@ class Transaction:
                 pre_hash = Hash(self.serialize_preimage(i).decode('hex'))
                 # der to string
                 order = ecdsa.ecdsa.generator_secp256k1.order()
-                r, s = ecdsa.util.sigdecode_der(sig.decode('hex'), order)
+                r, s = ecdsa.util.sigdecode_der(sig.decode('hex')[:-1], order)
                 sig_string = ecdsa.util.sigencode_string(r, s, order)
                 compressed = True
                 for recid in range(4):
@@ -582,6 +583,7 @@ class Transaction:
         self._inputs = d['inputs']
         self._outputs = [(x['type'], x['address'], x['value']) for x in d['outputs']]
         self.locktime = d['lockTime']
+        self.version = d['version']
         return d
 
     @classmethod
@@ -618,10 +620,10 @@ class Transaction:
             is_complete = len(signatures) == num_sig
             if is_complete:
                 pk_list = pubkeys
-                sig_list = [(sig + '01') for sig in signatures]
+                sig_list = signatures
             else:
                 pk_list = x_pubkeys
-                sig_list = [(sig + '01') if sig else NO_SIGNATURE for sig in x_signatures]
+                sig_list = [sig if sig else NO_SIGNATURE for sig in x_signatures]
         return pk_list, sig_list
 
     @classmethod
@@ -655,8 +657,8 @@ class Transaction:
             return push_script(redeem_script)
         elif _type == 'address':
             script += push_script(pubkeys[0])
-        else:
-            raise TypeError('Unknown txin type', _type)
+        elif _type == 'unknown':
+            return txin['scriptSig']
         return script
 
     @classmethod
@@ -667,6 +669,10 @@ class Transaction:
         elif txin['type'] == 'p2sh':
             pubkeys, x_pubkeys = self.get_sorted_pubkeys(txin)
             return multisig_script(pubkeys, txin['num_sig'])
+        elif txin['type'] == 'p2wpkh-p2sh':
+            pubkey = txin['pubkeys'][0]
+            pkh = bitcoin.hash_160(pubkey.decode('hex')).encode('hex')
+            return '76a9' + push_script(pkh) + '88ac'
         else:
             raise TypeError('Unknown txin type', _type)
 
@@ -681,11 +687,11 @@ class Transaction:
         # Script length, script, sequence
         s += var_int(len(script)/2)
         s += script
-        s += int_to_hex(txin.get('sequence', 0xffffffff), 4)
+        s += int_to_hex(txin.get('sequence', 0xffffffff - 1), 4)
         return s
 
     def set_rbf(self, rbf):
-        nSequence = 0xffffffff - (2 if rbf else 0)
+        nSequence = 0xffffffff - (2 if rbf else 1)
         for txin in self.inputs():
             txin['sequence'] = nSequence
 
@@ -703,7 +709,7 @@ class Transaction:
         return s
 
     def serialize_preimage(self, i):
-        nVersion = int_to_hex(1, 4)
+        nVersion = int_to_hex(self.version, 4)
         nHashType = int_to_hex(1, 4)
         nLocktime = int_to_hex(self.locktime, 4)
         inputs = self.inputs()
@@ -711,17 +717,13 @@ class Transaction:
         txin = inputs[i]
         if self.is_segwit_input(txin):
             hashPrevouts = Hash(''.join(self.serialize_outpoint(txin) for txin in inputs).decode('hex')).encode('hex')
-            hashSequence = Hash(''.join(int_to_hex(txin.get('sequence', 0xffffffff), 4) for txin in inputs).decode('hex')).encode('hex')
+            hashSequence = Hash(''.join(int_to_hex(txin.get('sequence', 0xffffffff - 1), 4) for txin in inputs).decode('hex')).encode('hex')
             hashOutputs = Hash(''.join(self.serialize_output(o) for o in outputs).decode('hex')).encode('hex')
             outpoint = self.serialize_outpoint(txin)
-            pubkey = txin['pubkeys'][0]
-            pkh = bitcoin.hash_160(pubkey.decode('hex')).encode('hex')
-            redeemScript = '00' + push_script(pkh)
-            scriptCode = push_script('76a9' + push_script(pkh) + '88ac')
-            script_hash = bitcoin.hash_160(redeemScript.decode('hex')).encode('hex')
-            scriptPubKey = 'a9' + push_script(script_hash) + '87'
+            preimage_script = self.get_preimage_script(txin)
+            scriptCode = var_int(len(preimage_script)/2) + preimage_script
             amount = int_to_hex(txin['value'], 8)
-            nSequence = int_to_hex(txin.get('sequence', 0xffffffff), 4)
+            nSequence = int_to_hex(txin.get('sequence', 0xffffffff - 1), 4)
             preimage = nVersion + hashPrevouts + hashSequence + outpoint + scriptCode + amount + nSequence + hashOutputs + nLocktime + nHashType
         else:
             txins = var_int(len(inputs)) + ''.join(self.serialize_input(txin, self.get_preimage_script(txin) if i==k else '') for k, txin in enumerate(inputs))
@@ -733,7 +735,7 @@ class Transaction:
         return any(self.is_segwit_input(x) for x in self.inputs())
 
     def serialize(self, estimate_size=False, witness=True):
-        nVersion = int_to_hex(1, 4)
+        nVersion = int_to_hex(self.version, 4)
         nLocktime = int_to_hex(self.locktime, 4)
         inputs = self.inputs()
         outputs = self.outputs()
@@ -780,7 +782,7 @@ class Transaction:
         return self.input_value() - self.output_value()
 
     def is_final(self):
-        return not any([x.get('sequence', 0xffffffff) < 0xffffffff - 1 for x in self.inputs()])
+        return not any([x.get('sequence', 0xffffffff - 1) < 0xffffffff - 1 for x in self.inputs()])
 
     @profiler
     def estimated_size(self):
@@ -829,8 +831,9 @@ class Transaction:
                     public_key = private_key.get_verifying_key()
                     sig = private_key.sign_digest_deterministic(pre_hash, hashfunc=hashlib.sha256, sigencode = ecdsa.util.sigencode_der)
                     assert public_key.verify_digest(sig, pre_hash, sigdecode = ecdsa.util.sigdecode_der)
-                    txin['signatures'][j] = sig.encode('hex')
+                    txin['signatures'][j] = sig.encode('hex') + '01'
                     txin['x_pubkeys'][j] = pubkey
+                    txin['pubkeys'][j] = pubkey # needed for fd keys
                     self._inputs[i] = txin
         print_error("is_complete", self.is_complete())
         self.raw = self.serialize()
